/***************************************************************************
                          kturingwnd.cpp  -  description
                             -------------------
    begin                : lun dic 2 2002
    copyright            : (C) 2002 by Beniamino Galvani
    email                : ben_gal@libero.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qtable.h>
#include <qcanvas.h>
#include <qcolor.h>
#include <qfont.h>
#include <iostream>
#include <kled.h>
#include <qbrush.h>
#include <qlcdnumber.h>
#include <qtimer.h>


#include "kturingwnd.h"

KTuringWnd::KTuringWnd(QWidget *parent, const char *name) : KTuringUi(parent,name){

  textOffset = 0;
  delay = 200;

  machine = new Machine();

  for(int c=0; c < stateTable->numCols() ; c++)
    stateTable->setColumnWidth(c,65);

  canvas = new QCanvas(tapeCanvasView->width(),tapeCanvasView->height());
  canvas->setBackgroundColor(blue);
  tapeCanvasView->setCanvas(canvas);
  tapeCanvasView->setVScrollBarMode(QScrollView::AlwaysOff);


  font = new QFont("Courier",30);
  metrix = new QFontMetrics(*font);
  text = new QCanvasText("", *font,canvas);

  text->setX(10);
  text->setY(5);
  text->setColor(yellow);
  text->show();

  rect = new QCanvasRectangle(canvas->width()/2-1,0,
                              0,canvas->height()-4,canvas);
  rect->setBrush(QBrush(red));
  rect->setZ(-1);
  rect->show();

  led->setColor(red);
  stateLcd->display(1);

  connect(loadButton, SIGNAL( pressed() ),  SLOT( load() ) );
  connect(stepButton, SIGNAL( pressed() ),  SLOT( step() ) );
  connect(stopButton, SIGNAL( pressed() ),  SLOT( stop() ) );
  connect(goButton,   SIGNAL( pressed() ),  SLOT( go() ) );
  connect(leftButton, SIGNAL( pressed() ),  SLOT( tapeLeft() ) );
  connect(rightButton, SIGNAL( pressed() ),  SLOT( tapeRight() ) );


}

KTuringWnd::~KTuringWnd(){}

void KTuringWnd::load(){

  state s;
  int r=0;

  machine = new Machine();

  while(1){
    bool inv_row = false;

    for(int c=0; c < 5; c++){
      if(!stateTable->text(r,c))
        inv_row = true;
    }

    if(inv_row)
      break;

    s.s = stateTable->item(r,0)->text().toInt();
    s.c =   stateTable->item(r,1)->text().at(0);
    s.nc =  stateTable->item(r,2)->text().at(0);

    QChar c = stateTable->item(r,3)->text().at(0);
    if(c == '1' || c == 'r' || c == 'R')
      s.dir = 1;
    else s.dir = 0;

    s.ns =  stateTable->item(r,4)->text().toInt();

    r++;
    machine->addState(&s);
  }

  if (r != 0){
    led->setColor(green);
    machine->setTape(tapeEdit->text());
    updateText();
    stateLcd->display(1);
  }

}

void KTuringWnd::step(){

  if(machine->finished())
    return;

  machine->doStep();
  updateText();
  stateLcd->display(machine->getCstate());

  if (machine->finished()){
    led->setColor(red);
    timer->stop();
  }
}

void KTuringWnd::stop(){

  timer->stop();

}


void KTuringWnd::go(){

/*  while(!machine->finished()){
    step();

  }
  led->setColor(red);
*/

  timer = new QTimer( this );
  connect( timer, SIGNAL(timeout()), this, SLOT(step()) );
  timer->start( delay, FALSE ); // 2 seconds timer

}

void KTuringWnd::updateText(){

  int cpos = machine->getCpos();
  QString tape(machine->getTape());
  int middle = canvas->width()/2;

  int abs = middle - metrix->width(tape.left(cpos));

//  if((abs >= 0) && (abs <= middle))
    text->setX(abs - textOffset);
//  else{
//    while (middle - metrix->width(tape.left(cpos)) < 0){
//      cpos--;
//      tape = tape.mid(1);
//    }
//  }
  rect->setX(canvas->width()/2 - textOffset);
  rect->setSize(metrix->width(tape.mid(cpos,1))+1,rect->height());
  text->setText(tape);
  canvas->update();

}

void KTuringWnd::tapeLeft(){
  textOffset -= 5;
  updateText();
}

void KTuringWnd::tapeRight(){
  textOffset += 5;
  updateText();
}

